@@ -1,5 +1,5 @@
 ;;; dgnushack.el --- a hack to set the load path for byte-compiling
-;; Copyright (C) 1994,95,96,97 Free Software Foundation, Inc.
+;; Copyright (C) 1994,95,96,97,98 Free Software Foundation, Inc.
 
 ;; Author: Lars Magne Ingebrigtsen <larsi@ifi.uio.no>
 ;; Version: 4.19
@@ -30,8 +30,9 @@
 
 (require 'cl)
 (require 'bytecomp)
+(push "~/lisp/custom" load-path)
 (push "." load-path)
-(require 'lpath)
+(load "./lpath.el")
 
 (defalias 'device-sound-enabled-p 'ignore)
 (defalias 'play-sound-file 'ignore)
@@ -47,26 +48,33 @@
     (fset 'x-defined-colors 'ignore)
     (fset 'read-color 'ignore)))
 
+(setq byte-compile-warnings
+      '(free-vars unresolved callargs redefine))
+
 (defun dgnushack-compile ()
   ;;(setq byte-compile-dynamic t)
-  (let ((files (directory-files "." nil ".el$"))
+  (unless (locate-library "cus-edit")
+    (error "You do not seem to have Custom installed.
+Fetch it from <URL:http://www.dina.kvl.dk/~abraham/custom/>.
+You also then need to add the following to the lisp/dgnushack.el file:
+
+     (push \"~/lisp/custom\" load-path)
+
+Modify to suit your needs."))
+  (let ((files (directory-files "." nil "^[^=].*\\.el$"))
 	(xemacs (string-match "XEmacs" emacs-version))
 	;;(byte-compile-generate-call-tree t)
-	byte-compile-warnings file elc)
+	file elc)
     (condition-case ()
 	(require 'w3-forms)
-      (error (setq files (delete "nnweb.el" files))))
+      (error (setq files (delete "nnweb.el" (delete "nnlistserv.el" files)))))
     (while (setq file (pop files))
-      (cond 
-       ((or (string= file "custom.el") (string= file "browse-url.el"))
-	(setq byte-compile-warnings nil))
-       (t
-	(setq byte-compile-warnings 
-	      '(free-vars unresolved callargs redefine obsolete))))
-      (when (or (not (member file '("gnus-xmas.el" "gnus-picon.el" 
-				    "messagexmas.el" "nnheaderxm.el"
-				    "smiley.el")))
-		xemacs)
+      (when (or (and (not xemacs)
+		     (not (member file '("gnus-xmas.el" "gnus-picon.el"
+					 "messagexmas.el" "nnheaderxm.el"
+					 "smiley.el" "x-overlay.el"))))
+		(and xemacs
+		     (not (member file '("md5.el")))))
 	(when (or (not (file-exists-p (setq elc (concat file "c"))))
 		  (file-newer-than-file-p file elc))
 	  (ignore-errors
@@ -76,5 +84,5 @@
   (require 'gnus)
   (byte-recompile-directory "." 0))
 
-;;; dgnushack.el ends here  
+;;; dgnushack.el ends here
 
