@@ -1,7 +1,8 @@
 ;;; gnus-salt.el --- alternate summary mode interfaces for Gnus
-;; Copyright (C) 1996,97 Free Software Foundation, Inc.
+;; Copyright (C) 1996,97,98 Free Software Foundation, Inc.
 
 ;; Author: Lars Magne Ingebrigtsen <larsi@ifi.uio.no>
+;; Keywords: news
 
 ;; This file is part of GNU Emacs.
 
@@ -24,6 +25,8 @@
 
 ;;; Code:
 
+(eval-when-compile (require 'cl))
+
 (require 'gnus)
 (require 'gnus-sum)
 
@@ -55,7 +58,7 @@
   :group 'gnus-summary-pick)
 
 (defcustom gnus-summary-pick-line-format
-  "%-5P %U\%R\%z\%I\%(%[%4L: %-20,20n%]%) %s\n"
+  "-5P %U\%R\%z\%I\%(%[%4L: %-20,20n%]%) %s\n"
   "*The format specification of the lines in pick buffers.
 It accepts the same format specs that `gnus-summary-line-format' does."
   :type 'string
@@ -68,25 +71,24 @@ It accepts the same format specs that `gnus-summary-line-format' does."
 (unless gnus-pick-mode-map
   (setq gnus-pick-mode-map (make-sparse-keymap))
 
-  (gnus-define-keys
-   gnus-pick-mode-map
-   "t" gnus-uu-mark-thread
-   "T" gnus-uu-unmark-thread
-   " " gnus-pick-next-page
-   "u" gnus-summary-unmark-as-processable
-   "U" gnus-summary-unmark-all-processable
-   "v" gnus-uu-mark-over
-   "r" gnus-uu-mark-region
-   "R" gnus-uu-unmark-region
-   "e" gnus-uu-mark-by-regexp
-   "E" gnus-uu-mark-by-regexp
-   "b" gnus-uu-mark-buffer
-   "B" gnus-uu-unmark-buffer
-   "." gnus-pick-article
-   gnus-down-mouse-2 gnus-pick-mouse-pick-region
-   ;;gnus-mouse-2 gnus-pick-mouse-pick
-   "X" gnus-pick-start-reading
-   "\r" gnus-pick-start-reading))
+  (gnus-define-keys gnus-pick-mode-map
+    "t" gnus-uu-mark-thread
+    "T" gnus-uu-unmark-thread
+    " " gnus-pick-next-page
+    "u" gnus-summary-unmark-as-processable
+    "U" gnus-summary-unmark-all-processable
+    "v" gnus-uu-mark-over
+    "r" gnus-uu-mark-region
+    "R" gnus-uu-unmark-region
+    "e" gnus-uu-mark-by-regexp
+    "E" gnus-uu-mark-by-regexp
+    "b" gnus-uu-mark-buffer
+    "B" gnus-uu-unmark-buffer
+    "." gnus-pick-article
+    gnus-down-mouse-2 gnus-pick-mouse-pick-region
+    ;;gnus-mouse-2 gnus-pick-mouse-pick
+    "X" gnus-pick-start-reading
+    "\r" gnus-pick-start-reading))
 
 (defun gnus-pick-make-menu-bar ()
   (unless (boundp 'gnus-pick-menu)
@@ -131,7 +133,7 @@ It accepts the same format specs that `gnus-summary-line-format' does."
       (when (gnus-visual-p 'pick-menu 'menu)
 	(gnus-pick-make-menu-bar))
       (gnus-add-minor-mode 'gnus-pick-mode " Pick" gnus-pick-mode-map)
-      (run-hooks 'gnus-pick-mode-hook))))
+      (gnus-run-hooks 'gnus-pick-mode-hook))))
 
 (defun gnus-pick-setup-message ()
   "Make Message do the right thing on exit."
@@ -202,7 +204,7 @@ This must be bound to a button-down mouse event."
          (start-line (1+ (count-lines 1 start-point)))
 	 (start-window (posn-window start-posn))
 	 (start-frame (window-frame start-window))
-	 (bounds (window-edges start-window))
+	 (bounds (gnus-window-edges start-window))
 	 (top (nth 1 bounds))
 	 (bottom (if (window-minibuffer-p start-window)
 		     (nth 3 bounds)
@@ -334,7 +336,7 @@ This must be bound to a button-down mouse event."
       (when (gnus-visual-p 'binary-menu 'menu)
 	(gnus-binary-make-menu-bar))
       (gnus-add-minor-mode 'gnus-binary-mode " Binary" gnus-binary-mode-map)
-      (run-hooks 'gnus-binary-mode-hook))))
+      (gnus-run-hooks 'gnus-binary-mode-hook))))
 
 (defun gnus-binary-display-article (article &optional all-header)
   "Run ARTICLE through the binary decode functions."
@@ -443,12 +445,8 @@ Two predefined functions are available:
 (defun gnus-tree-mode ()
   "Major mode for displaying thread trees."
   (interactive)
-  (setq gnus-tree-mode-line-format-spec
-	(gnus-parse-format gnus-tree-mode-line-format
-			   gnus-summary-mode-line-format-alist))
-  (setq gnus-tree-line-format-spec
-	(gnus-parse-format gnus-tree-line-format
-			   gnus-tree-line-format-alist t))
+  (gnus-set-format 'tree-mode)
+  (gnus-set-format 'tree t)
   (when (gnus-visual-p 'tree-menu 'menu)
     (gnus-tree-make-menu-bar))
   (kill-all-local-variables)
@@ -463,7 +461,7 @@ Two predefined functions are available:
     (gnus-set-work-buffer)
     (gnus-tree-node-insert (make-mail-header "") nil)
     (setq gnus-tree-node-length (1- (point))))
-  (run-hooks 'gnus-tree-mode-hook))
+  (gnus-run-hooks 'gnus-tree-mode-hook))
 
 (defun gnus-tree-read-summary-keys (&optional arg)
   "Read a summary buffer key sequence and execute it."
@@ -798,8 +796,7 @@ Two predefined functions are available:
   (gnus-get-tree-buffer))
 
 (defun gnus-tree-close (group)
-					;(gnus-kill-buffer gnus-tree-buffer)
-  )
+  (gnus-kill-buffer gnus-tree-buffer))
 
 (defun gnus-highlight-selected-tree (article)
   "Highlight the selected article in the tree."
@@ -958,7 +955,7 @@ The following commands are available:
   (buffer-disable-undo (current-buffer))
   (setq buffer-read-only t)
   (make-local-variable 'gnus-carpal-attached-buffer)
-  (run-hooks 'gnus-carpal-mode-hook))
+  (gnus-run-hooks 'gnus-carpal-mode-hook))
 
 (defun gnus-carpal-setup-buffer (type)
   (let ((buffer (symbol-value (intern (format "gnus-carpal-%s-buffer" type)))))
