@@ -1,5 +1,5 @@
-1;;; gnus-score.el --- scoring code for Gnus
-;; Copyright (C) 1995,96,97 Free Software Foundation, Inc.
+;;; gnus-score.el --- scoring code for Gnus
+;; Copyright (C) 1995,96,97,98 Free Software Foundation, Inc.
 
 ;; Author: Per Abrahamsen <amanda@iesd.auc.dk>
 ;;	Lars Magne Ingebrigtsen <larsi@ifi.uio.no>
@@ -26,6 +26,8 @@
 
 ;;; Code:
 
+(eval-when-compile (require 'cl))
+
 (require 'gnus)
 (require 'gnus-sum)
 (require 'gnus-range)
@@ -105,7 +107,11 @@ See the documentation to these functions for more information.
 
 This variable can also be a list of functions to be called.  Each
 function should either return a list of score files, or a list of
-score alists."
+score alists.
+
+If functions other than these pre-defined functions are used,
+the `a' symbolic prefix to the score commands will always use
+\"all.SCORE\"."
   :group 'gnus-score-files
   :type '(radio (function-item gnus-score-find-single)
 		(function-item gnus-score-find-hierarchical)
@@ -134,7 +140,8 @@ will be expired along with non-matching score entries."
 (defcustom gnus-orphan-score nil
   "*All orphans get this score added.  Set in the score file."
   :group 'gnus-score-default
-  :type 'integer)
+  :type '(choice (const nil)
+		 integer))
 
 (defcustom gnus-decay-scores nil
   "*If non-nil, decay non-permanent scores."
@@ -192,8 +199,8 @@ It can be:
   :type '(choice string
 		 (repeat (choice string
 				 (cons regexp (repeat file))
-				 function))
-		 function))
+				 (function :value fun)))
+		 (function :value fun)))
 
 (defcustom gnus-home-adapt-file nil
   "Variable to control where new adaptive score entries are to go.
@@ -203,8 +210,8 @@ This variable allows the same syntax as `gnus-home-score-file'."
   :type '(choice string
 		 (repeat (choice string
 				 (cons regexp (repeat file))
-				 function))
-		 function))
+				 (function :value fun)))
+		 (function :value fun)))
 
 (defcustom gnus-default-adaptive-score-alist
   '((gnus-kill-file-mark)
@@ -213,7 +220,7 @@ This variable allows the same syntax as `gnus-home-score-file'."
     (gnus-catchup-mark (subject -10))
     (gnus-killed-mark (from -1) (subject -20))
     (gnus-del-mark (from -2) (subject -15)))
-"Alist of marks and scores."
+"*Alist of marks and scores."
 :group 'gnus-score-adapt
 :type '(repeat (cons (symbol :tag "Mark")
 		     (repeat (list (choice :tag "Header"
@@ -242,7 +249,7 @@ This variable allows the same syntax as `gnus-home-score-file'."
     "being" "current" "back" "still" "go" "point" "value" "each" "did"
     "both" "true" "off" "say" "another" "state" "might" "under" "start"
     "try" "re")
-  "Default list of words to be ignored when doing adaptive word scoring."
+  "*Default list of words to be ignored when doing adaptive word scoring."
   :group 'gnus-score-adapt
   :type '(repeat string))
 
@@ -251,11 +258,16 @@ This variable allows the same syntax as `gnus-home-score-file'."
     (,gnus-catchup-mark . -10)
     (,gnus-killed-mark . -20)
     (,gnus-del-mark . -15))
-"Alist of marks and scores."
+"*Alist of marks and scores."
 :group 'gnus-score-adapt
 :type '(repeat (cons (character :tag "Mark")
 		     (integer :tag "Score"))))
 
+(defcustom gnus-adaptive-word-minimum nil
+  "If a number, this is the minimum score value that can be assigned to a word."
+  :group 'gnus-score-adapt
+  :type '(choice (const nil) integer))
+
 (defcustom gnus-score-mimic-keymap nil
   "*Have the score entry functions pretend that they are a keymap."
   :group 'gnus-score-default
@@ -305,7 +317,8 @@ If nil, the user will be asked for a header."
 		 (const :tag "xref" x)
 		 (const :tag "lines" l)
 		 (const :tag "date" d)
-		 (const :tag "followup" f)))
+		 (const :tag "followup" f)
+		 (const :tag "ask" nil)))
 
 (defcustom gnus-score-default-type nil
   "Default match type when entering new scores.
@@ -334,7 +347,8 @@ If nil, the user will be asked for a match type."
 		 (const :tag "this date" n)
 		 (const :tag "less than number" <)
 		 (const :tag "greater than number" >)
-		 (const :tag "equal than number" =)))
+		 (const :tag "equal than number" =)
+		 (const :tag "ask" nil)))
 
 (defcustom gnus-score-default-fold nil
   "Use case folding for new score file entries iff not nil."
@@ -362,6 +376,11 @@ If nil, the user will be asked for a duration."
   :group 'gnus-score-files
   :type 'function)
 
+(defcustom gnus-score-thread-simplify nil
+  "If non-nil, subjects will simplified as in threading."
+  :group 'gnus-score-various
+  :type 'boolean) 
+
 
 
 ;; Internal variables.
@@ -447,13 +466,13 @@ of the last successful match.")
 ;; Much modification of the kill (ahem, score) code and lots of the
 ;; functions are written by Per Abrahamsen <amanda@iesd.auc.dk>.
 
-(defun gnus-summary-lower-score (&optional score)
+(defun gnus-summary-lower-score (&optional score symp)
   "Make a score entry based on the current article.
 The user will be prompted for header to score on, match type,
 permanence, and the string to be used.  The numerical prefix will be
 used as score."
-  (interactive "P")
-  (gnus-summary-increase-score (- (gnus-score-default score))))
+  (interactive (gnus-interactive "P\ny"))
+  (gnus-summary-increase-score (- (gnus-score-default score)) symp))
 
 (defun gnus-score-kill-help-buffer ()
   (when (get-buffer "*Score Help*")
@@ -461,13 +480,12 @@ used as score."
     (when gnus-score-help-winconf
       (set-window-configuration gnus-score-help-winconf))))
 
-(defun gnus-summary-increase-score (&optional score)
+(defun gnus-summary-increase-score (&optional score symp)
   "Make a score entry based on the current article.
 The user will be prompted for header to score on, match type,
 permanence, and the string to be used.  The numerical prefix will be
 used as score."
-  (interactive "P")
-  (gnus-set-global-variables)
+  (interactive (gnus-interactive "P\ny"))
   (let* ((nscore (gnus-score-default score))
 	 (prefix (if (< nscore 0) ?L ?I))
 	 (increase (> nscore 0))
@@ -496,6 +514,7 @@ used as score."
 	    (?< < "less than number" number)
 	    (?> > "greater than number" number)
 	    (?= = "equal to number" number)))
+	 (current-score-file gnus-current-score-file)
 	 (char-to-perm
 	  (list (list ?t (current-time-string) "temporary")
 		'(?p perm "permanent") '(?i now "immediate")))
@@ -613,6 +632,21 @@ used as score."
     (when (memq type '(r R regexp Regexp))
       (setq match (regexp-quote match)))
 
+    ;; Change score file to the "all.SCORE" file.
+    (when (eq symp 'a)
+      (save-excursion
+	(set-buffer gnus-summary-buffer)
+	(gnus-score-load-file
+	 ;; This is a kludge; yes...
+	 (cond
+	  ((eq gnus-score-find-score-files-function
+	       'gnus-score-find-hierarchical)
+	   (gnus-score-file-name ""))
+	  ((eq gnus-score-find-score-files-function 'gnus-score-find-single)
+	   current-score-file)
+	  (t
+	   (gnus-score-file-name "all"))))))
+    
     (gnus-summary-score-entry
      (nth 1 entry)			; Header
      match				; Match
@@ -622,7 +656,12 @@ used as score."
 	 nil
        temporary)
      (not (nth 3 entry)))		; Prompt
-    ))
+
+    (when (eq symp 'a)
+      ;; We change the score file back to the previous one.
+      (save-excursion
+	(set-buffer gnus-summary-buffer)
+	(gnus-score-load-file current-score-file)))))
 
 (defun gnus-score-insert-help (string alist idx)
   (setq gnus-score-help-winconf (current-window-configuration))
@@ -783,7 +822,7 @@ If optional argument `SILENT' is nil, show effect of score entry."
 				  (or (nth 1 new)
 				      gnus-score-interactive-default-score)))
 	  ;; Nope, we have to add a new elem.
-	  (gnus-score-set header (if old (cons new old) (list new))))
+	  (gnus-score-set header (if old (cons new old) (list new)) nil t))
 	(gnus-score-set 'touched '(t))))
 
     ;; Score the current buffer.
@@ -933,7 +972,7 @@ SCORE is the score to add."
 	     "references" id 's
 	     score (current-time-string))))))))
 
-(defun gnus-score-set (symbol value &optional alist)
+(defun gnus-score-set (symbol value &optional alist warn)
   ;; Set SYMBOL to VALUE in ALIST.
   (let* ((alist
 	  (or alist
@@ -942,7 +981,8 @@ SCORE is the score to add."
 	 (entry (assoc symbol alist)))
     (cond ((gnus-score-get 'read-only alist)
 	   ;; This is a read-only score file, so we do nothing.
-	   )
+	   (when warn
+	     (gnus-message 4 "Note: read-only score file; entry discarded")))
 	  (entry
 	   (setcdr entry value))
 	  ((null alist)
@@ -954,14 +994,12 @@ SCORE is the score to add."
 (defun gnus-summary-raise-score (n)
   "Raise the score of the current article by N."
   (interactive "p")
-  (gnus-set-global-variables)
   (gnus-summary-set-score (+ (gnus-summary-article-score)
 			     (or n gnus-score-interactive-default-score ))))
 
 (defun gnus-summary-set-score (n)
   "Set the score of the current article to N."
   (interactive "p")
-  (gnus-set-global-variables)
   (save-excursion
     (gnus-summary-show-thread)
     (let ((buffer-read-only nil))
@@ -980,7 +1018,6 @@ SCORE is the score to add."
 (defun gnus-summary-current-score ()
   "Return the score of the current article."
   (interactive)
-  (gnus-set-global-variables)
   (gnus-message 1 "%s" (gnus-summary-article-score)))
 
 (defun gnus-score-change-score-file (file)
@@ -994,21 +1031,21 @@ SCORE is the score to add."
 (defun gnus-score-edit-current-scores (file)
   "Edit the current score alist."
   (interactive (list gnus-current-score-file))
-  (gnus-set-global-variables)
-  (let ((winconf (current-window-configuration)))
-    (when (buffer-name gnus-summary-buffer)
-      (gnus-score-save))
-    (gnus-make-directory (file-name-directory file))
-    (setq gnus-score-edit-buffer (find-file-noselect file))
-    (gnus-configure-windows 'edit-score)
-    (select-window (get-buffer-window gnus-score-edit-buffer))
-    (gnus-score-mode)
-    (setq gnus-score-edit-exit-function 'gnus-score-edit-done)
-    (make-local-variable 'gnus-prev-winconf)
-    (setq gnus-prev-winconf winconf))
-  (gnus-message
-   4 (substitute-command-keys
-      "\\<gnus-score-mode-map>\\[gnus-score-edit-exit] to save edits")))
+  (if (not gnus-current-score-file)
+      (error "No current score file")
+    (let ((winconf (current-window-configuration)))
+      (when (buffer-name gnus-summary-buffer)
+	(gnus-score-save))
+      (gnus-make-directory (file-name-directory file))
+      (setq gnus-score-edit-buffer (find-file-noselect file))
+      (gnus-configure-windows 'edit-score)
+      (gnus-score-mode)
+      (setq gnus-score-edit-exit-function 'gnus-score-edit-done)
+      (make-local-variable 'gnus-prev-winconf)
+      (setq gnus-prev-winconf winconf))
+    (gnus-message
+     4 (substitute-command-keys
+	"\\<gnus-score-mode-map>\\[gnus-score-edit-exit] to save edits"))))
 
 (defun gnus-score-edit-file (file)
   "Edit a score file."
@@ -1083,30 +1120,35 @@ SCORE is the score to add."
 	  (decay (car (gnus-score-get 'decay alist)))
 	  (eval (car (gnus-score-get 'eval alist))))
       ;; Perform possible decays.
-      (when gnus-decay-scores
-	(when (or (not decay)
-		  (gnus-decay-scores alist decay))
-	  (gnus-score-set 'touched '(t) alist)
-	  (gnus-score-set 'decay (list (gnus-time-to-day (current-time))))))
+      (when (and gnus-decay-scores
+		 (or cached (file-exists-p file))
+		 (or (not decay)
+		     (gnus-decay-scores alist decay)))
+	(gnus-score-set 'touched '(t) alist)
+	(gnus-score-set 'decay (list (gnus-time-to-day (current-time)))))
       ;; We do not respect eval and files atoms from global score
       ;; files.
-      (and files (not global)
-	   (setq lists (apply 'append lists
-			      (mapcar (lambda (file)
-					(gnus-score-load-file file))
-				      (if adapt-file (cons adapt-file files)
-					files)))))
-      (and eval (not global) (eval eval))
+      (when (and files (not global))
+	(setq lists (apply 'append lists
+			   (mapcar (lambda (file)
+				     (gnus-score-load-file file))
+				   (if adapt-file (cons adapt-file files)
+				     files)))))
+      (when (and eval (not global))
+	(eval eval))
       ;; We then expand any exclude-file directives.
       (setq gnus-scores-exclude-files
 	    (nconc
-	     (mapcar
-	      (lambda (sfile)
-		(expand-file-name sfile (file-name-directory file)))
-	      exclude-files)
+	     (apply
+	      'nconc
+	      (mapcar
+	       (lambda (sfile)
+		 (list
+		  (expand-file-name sfile (file-name-directory file))
+		  (expand-file-name sfile gnus-kill-files-directory)))
+	       exclude-files))
 	     gnus-scores-exclude-files))
-      (if (not local)
-	  ()
+      (when local
 	(save-excursion
 	  (set-buffer gnus-summary-buffer)
 	  (while local
@@ -1360,7 +1402,8 @@ SCORE is the score to add."
 	  (save-excursion
 	    (set-buffer (get-buffer-create "*Headers*"))
 	    (buffer-disable-undo (current-buffer))
-	    (message-clone-locals gnus-summary-buffer)
+	    (when (gnus-buffer-live-p gnus-summary-buffer)
+	      (message-clone-locals gnus-summary-buffer))
 
 	    ;; Set the global variant of this variable.
 	    (setq gnus-current-score-file current-score-file)
@@ -1807,6 +1850,8 @@ SCORE is the score to add."
   ;; Insert the unique article headers in the buffer.
   (let ((gnus-score-index (nth 1 (assoc header gnus-header-index)))
 	;; gnus-score-index is used as a free variable.
+        (simplify (and gnus-score-thread-simplify
+                       (string= "subject" header)))
 	alike last this art entries alist articles
 	fuzzies arts words kill)
 
@@ -1822,6 +1867,8 @@ SCORE is the score to add."
     (erase-buffer)
     (while (setq art (pop articles))
       (setq this (aref (car art) gnus-score-index))
+      (if simplify
+        (setq this (gnus-map-function gnus-simplify-subject-functions this)))
       (if (equal last this)
 	  ;; O(N*H) cons-cells used here, where H is the number of
 	  ;; headers.
@@ -1847,7 +1894,6 @@ SCORE is the score to add."
 	    entries (assoc header alist))
       (while (cdr entries)		;First entry is the header index.
 	(let* ((kill (cadr entries))
-	       (match (nth 0 kill))
 	       (type (or (nth 3 kill) 's))
 	       (score (or (nth 1 kill) gnus-score-interactive-default-score))
 	       (date (nth 2 kill))
@@ -1855,6 +1901,12 @@ SCORE is the score to add."
 	       (mt (aref (symbol-name type) 0))
 	       (case-fold-search (not (memq mt '(?R ?S ?E ?F))))
 	       (dmt (downcase mt))
+               ; Assume user already simplified regexp and fuzzies
+	       (match (if (and simplify (not (memq dmt '(?f ?r))))
+                          (gnus-map-function
+                           gnus-simplify-subject-functions
+                           (nth 0 kill))
+                        (nth 0 kill)))
 	       (search-func
 		(cond ((= dmt ?r) 're-search-forward)
 		      ((or (= dmt ?e) (= dmt ?s) (= dmt ?f)) 'search-forward)
@@ -2059,6 +2111,7 @@ SCORE is the score to add."
       (set-buffer gnus-summary-buffer)
       (gnus-score-load-file
        (or gnus-newsgroup-adaptive-score-file
+	   (gnus-home-score-file gnus-newsgroup-name t)
 	   (gnus-score-file-name
 	    gnus-newsgroup-name gnus-adaptive-file-suffix))))
     ;; Perform ordinary line scoring.
@@ -2069,7 +2122,7 @@ SCORE is the score to add."
 	       (alist malist)
 	       (date (current-time-string))
 	       (data gnus-newsgroup-data)
-	       elem headers match)
+	       elem headers match func)
 	  ;; First we transform the adaptive rule alist into something
 	  ;; that's faster to process.
 	  (while malist
@@ -2078,19 +2131,21 @@ SCORE is the score to add."
 	      (setcar elem (symbol-value (car elem))))
 	    (setq elem (cdr elem))
 	    (while elem
-	      (setcdr (car elem)
-		      (cons (if (eq (caar elem) 'followup)
-				"references"
-			      (symbol-name (caar elem)))
-			    (cdar elem)))
-	      (setcar (car elem)
-		      `(lambda (h)
-			 (,(intern
+	      (when (fboundp
+		     (setq func
+			   (intern
 			    (concat "mail-header-"
 				    (if (eq (caar elem) 'followup)
 					"message-id"
-				      (downcase (symbol-name (caar elem))))))
-			  h)))
+				      (downcase (symbol-name (caar elem))))))))
+		(setcdr (car elem)
+			(cons (if (eq (caar elem) 'followup)
+				  "references"
+				(symbol-name (caar elem)))
+			      (cdar elem)))
+		(setcar (car elem)
+			`(lambda (h)
+			   (,func h))))
 	      (setq elem (cdr elem)))
 	    (setq malist (cdr malist)))
 	  ;; Then we score away.
@@ -2151,7 +2206,11 @@ SCORE is the score to add."
 		      ;; Put the word and score into the hashtb.
 		      (setq val (gnus-gethash (setq word (match-string 0))
 					      hashtb))
-		      (gnus-sethash word (+ (or val 0) score) hashtb))
+		      (setq val (+ score (or val 0)))
+		      (if (and gnus-adaptive-word-minimum
+			       (< val gnus-adaptive-word-minimum))
+			  (setq val gnus-adaptive-word-minimum))
+		      (gnus-sethash word val hashtb))
 		    (erase-buffer))))
 	    (set-syntax-table syntab))
 	  ;; Make all the ignorable words ignored.
@@ -2541,7 +2600,7 @@ Destroys the current buffer."
 	    files)))
       (mapcar
        (lambda (f) (cdr f))
-       (sort alist (lambda (f1 f2) (< (car f1) (car f2))))))))
+       (sort alist 'car-less-than-car)))))
 
 (defun gnus-score-find-alist (group)
   "Return list of score files for GROUP.
@@ -2578,57 +2637,58 @@ The list is determined from the variable gnus-score-file-alist."
   (let ((funcs gnus-score-find-score-files-function)
 	(group (or group gnus-newsgroup-name))
 	score-files)
-    ;; Make sure funcs is a list.
-    (and funcs
-	 (not (listp funcs))
-	 (setq funcs (list funcs)))
-    ;; Get the initial score files for this group.
-    (when funcs
-      (setq score-files (nreverse (gnus-score-find-alist group))))
-    ;; Add any home adapt files.
-    (let ((home (gnus-home-score-file group t)))
-      (when home
-	(push home score-files)
-	(setq gnus-newsgroup-adaptive-score-file home)))
-    ;; Check whether there is a `adapt-file' group parameter.
-    (let ((param-file (gnus-group-find-parameter group 'adapt-file)))
-      (when param-file
-	(push param-file score-files)
-	(setq gnus-newsgroup-adaptive-score-file param-file)))
-    ;; Go through all the functions for finding score files (or actual
-    ;; scores) and add them to a list.
-    (while funcs
-      (when (gnus-functionp (car funcs))
-	(setq score-files
-	      (nconc score-files (nreverse (funcall (car funcs) group)))))
-      (setq funcs (cdr funcs)))
-    ;; Add any home score files.
-    (let ((home (gnus-home-score-file group)))
-      (when home
-	(push home score-files)))
-    ;; Check whether there is a `score-file' group parameter.
-    (let ((param-file (gnus-group-find-parameter group 'score-file)))
-      (when param-file
-	(push param-file score-files)))
-    ;; Expand all files names.
-    (let ((files score-files))
-      (while files
-	(when (stringp (car files))
-	  (setcar files (expand-file-name
-			 (car files) gnus-kill-files-directory)))
-	(pop files)))
-    (setq score-files (nreverse score-files))
-    ;; Remove any duplicate score files.
-    (while (and score-files
-		(member (car score-files) (cdr score-files)))
-      (pop score-files))
-    (let ((files score-files))
-      (while (cdr files)
- 	(if (member (cadr files) (cddr files))
- 	    (setcdr files (cddr files))
- 	  (pop files))))
-    ;; Do the scoring if there are any score files for this group.
-    score-files))
+    (when group
+      ;; Make sure funcs is a list.
+      (and funcs
+	   (not (listp funcs))
+	   (setq funcs (list funcs)))
+      ;; Get the initial score files for this group.
+      (when funcs
+	(setq score-files (nreverse (gnus-score-find-alist group))))
+      ;; Add any home adapt files.
+      (let ((home (gnus-home-score-file group t)))
+	(when home
+	  (push home score-files)
+	  (setq gnus-newsgroup-adaptive-score-file home)))
+      ;; Check whether there is a `adapt-file' group parameter.
+      (let ((param-file (gnus-group-find-parameter group 'adapt-file)))
+	(when param-file
+	  (push param-file score-files)
+	  (setq gnus-newsgroup-adaptive-score-file param-file)))
+      ;; Go through all the functions for finding score files (or actual
+      ;; scores) and add them to a list.
+      (while funcs
+	(when (gnus-functionp (car funcs))
+	  (setq score-files
+		(nconc score-files (nreverse (funcall (car funcs) group)))))
+	(setq funcs (cdr funcs)))
+      ;; Add any home score files.
+      (let ((home (gnus-home-score-file group)))
+	(when home
+	  (push home score-files)))
+      ;; Check whether there is a `score-file' group parameter.
+      (let ((param-file (gnus-group-find-parameter group 'score-file)))
+	(when param-file
+	  (push param-file score-files)))
+      ;; Expand all files names.
+      (let ((files score-files))
+	(while files
+	  (when (stringp (car files))
+	    (setcar files (expand-file-name
+			   (car files) gnus-kill-files-directory)))
+	  (pop files)))
+      (setq score-files (nreverse score-files))
+      ;; Remove any duplicate score files.
+      (while (and score-files
+		  (member (car score-files) (cdr score-files)))
+	(pop score-files))
+      (let ((files score-files))
+	(while (cdr files)
+	  (if (member (cadr files) (cddr files))
+	      (setcdr files (cddr files))
+	    (pop files))))
+      ;; Do the scoring if there are any score files for this group.
+      score-files)))
 
 (defun gnus-possibly-score-headers (&optional trace)
   "Do scoring if scoring is required."
@@ -2644,8 +2704,7 @@ The list is determined from the variable gnus-score-file-alist."
       ((or (null newsgroup)
 	   (string-equal newsgroup ""))
        ;; The global score file is placed at top of the directory.
-       (expand-file-name
-	suffix gnus-kill-files-directory))
+       (expand-file-name suffix gnus-kill-files-directory))
       ((gnus-use-long-file-name 'not-score)
        ;; Append ".SCORE" to newsgroup name.
        (expand-file-name (concat (gnus-newsgroup-savable-name newsgroup)
@@ -2759,6 +2818,63 @@ If ADAPT, return the home adaptive file instead."
     ;; Return whether this score file needs to be saved.  By Je-haysuss!
     updated))
 
+(defun gnus-score-regexp-bad-p (regexp)
+  "Test whether REGEXP is safe for Gnus scoring.
+A regexp is unsafe if it matches newline or a buffer boundary.
+
+If the regexp is good, return nil.  If the regexp is bad, return a
+cons cell (SYM . STRING), where the symbol SYM is `new' or `bad'.
+In the `new' case, the string is a safe replacement for REGEXP.
+In the `bad' case, the string is a unsafe subexpression of REGEXP,
+and we do not have a simple replacement to suggest.
+
+See `(Gnus)Scoring Tips' for examples of good regular expressions."
+  (let (case-fold-search)
+    (and
+     ;; First, try a relatively fast necessary condition.
+     ;; Notice ranges (like [^:] or [\t-\r]), \s>, \Sw, \W, \', \`:
+     (string-match "\n\\|\\\\[SsW`']\\|\\[\\^\\|[\0-\n]-" regexp)
+     ;; Now break the regexp into tokens, and check each:
+     (let ((tail regexp)		; remaining regexp to check
+	   tok				; current token
+	   bad				; nil, or bad subexpression
+	   new				; nil, or replacement regexp so far
+	   end)				; length of current token
+       (while (and (not bad)
+		   (string-match
+		    "\\`\\(\\\\[sS]?.\\|\\[\\^?]?[^]]*]\\|[^\\]\\)"
+		    tail))
+	 (setq end (match-end 0)
+	       tok (substring tail 0 end)
+	       tail (substring tail end))
+	 (if;; Is token `bad' (matching newline or buffer ends)?
+	     (or (member tok '("\n" "\\W" "\\`" "\\'"))
+		 ;; This next handles "[...]", "\\s.", and "\\S.":
+		 (and (> end 2) (string-match tok "\n")))
+	     (let ((newtok
+		    ;; Try to suggest a replacement for tok ...
+		    (cond ((string-equal tok "\\`") "^") ; or "\\(^\\)"
+			  ((string-equal tok "\\'") "$") ; or "\\($\\)"
+			  ((string-match "\\[\\^" tok) ; very common
+			   (concat (substring tok 0 -1) "\n]")))))
+	       (if newtok
+		   (setq new
+			 (concat
+			  (or new
+			      ;; good prefix so far:
+			      (substring regexp 0 (- (+ (length tail) end))))
+			  newtok))
+		 ;; No replacement idea, so give up:
+		 (setq bad tok)))
+	   ;; tok is good, may need to extend new
+	   (and new (setq new (concat new tok)))))
+       ;; Now return a value:
+       (cond
+	(bad (cons 'bad bad))
+	(new (cons 'new new))
+	;; or nil
+	)))))
+
 (provide 'gnus-score)
 
 ;;; gnus-score.el ends here
