@@ -1,5 +1,5 @@
 ;;; messagexmas.el --- XEmacs extensions to message
-;; Copyright (C) 1996,97 Free Software Foundation, Inc.
+;; Copyright (C) 1996,97,98 Free Software Foundation, Inc.
 
 ;; Author: Lars Magne Ingebrigtsen <larsi@ifi.uio.no>
 ;; Keywords: mail, news
@@ -43,7 +43,7 @@ If it is non-nil, it must be a toolbar.  The five legal values are
 `default-toolbar', `top-toolbar', `bottom-toolbar',
 `right-toolbar', and `left-toolbar'.")
 
-(defvar message-toolbar 
+(defvar message-toolbar
   '([message-spell ispell-message t "Spell"]
     [message-help (Info-goto-node "(Message)Top") t "Message help"])
   "The message buffer toolbar.")
@@ -93,13 +93,12 @@ If it is non-nil, it must be a toolbar.  The five legal values are
 (fset 'message-exchange-point-and-mark 'message-xmas-exchange-point-and-mark)
 
 (defun message-xmas-maybe-fontify ()
-  (when (and (featurep 'font-lock)
-	     font-lock-auto-fontify)
-    (turn-on-font-lock)))
+  (when (featurep 'font-lock)
+    (font-lock-set-defaults)))
 
 (defun message-xmas-make-caesar-translation-table (n)
   "Create a rot table with offset N."
-  (let ((i -1) 
+  (let ((i -1)
 	(table (make-string 256 0))
 	(a (char-int ?a))
 	(A (char-int ?A)))
