@@ -1,5 +1,5 @@
 ;;; messcompat.el --- making message mode compatible with mail mode
-;; Copyright (C) 1996,97 Free Software Foundation, Inc.
+;; Copyright (C) 1996,97,98 Free Software Foundation, Inc.
 
 ;; Author: Lars Magne Ingebrigtsen <larsi@ifi.uio.no>
 ;; Keywords: mail, news
@@ -24,7 +24,7 @@
 ;;; Commentary:
 
 ;; This file tries to provide backward compatability with sendmail.el
-;; for Message mode.  It should be used by simply adding 
+;; for Message mode.  It should be used by simply adding
 ;;
 ;; (require 'messcompat)
 ;;
@@ -56,10 +56,11 @@ nil means let mailer mail back a message to report errors.")
   "Normal hook, run each time a new outgoing message is initialized.
 The function `message-setup' runs this hook.")
 
-(defvar message-mode-hook mail-mode-hook
-  "Hook run in message mode buffers.")
+(if (boundp 'mail-mode-hook)
+    (defvar message-mode-hook mail-mode-hook
+      "Hook run in message mode buffers."))
 
-(defvar message-indentation-spaces mail-indentation-spaces 
+(defvar message-indentation-spaces mail-indentation-spaces
   "*Number of spaces to insert at the beginning of each cited line.
 Used by `message-yank-original' via `message-yank-cite'.")
 
@@ -69,7 +70,7 @@ If t, the `message-signature-file' file will be inserted instead.
 If a function, the result from the function will be used instead.
 If a form, the result from the form will be used instead.")
 
-;;;###autoload
+;; Deleted the autoload cookie because this crashes in loaddefs.el.
 (defvar message-signature-file mail-signature-file
   "*File containing the text inserted at end of message. buffer.")
 
