@@ -1,5 +1,5 @@
 ;;; nnfolder.el --- mail folder access for Gnus
-;; Copyright (C) 1995,96,97 Free Software Foundation, Inc.
+;; Copyright (C) 1995,96,97,98 Free Software Foundation, Inc.
 
 ;; Author: Scott Byer <byer@mv.us.adobe.com>
 ;;	Lars Magne Ingebrigtsen <larsi@ifi.uio.no>
@@ -283,8 +283,10 @@ time saver for large mailboxes.")
 (deffoo nnfolder-request-list (&optional server)
   (nnfolder-possibly-change-group nil server)
   (save-excursion
-    (nnmail-find-file nnfolder-active-file)
-    (setq nnfolder-group-alist (nnmail-get-active))
+    (let ((nnmail-file-coding-system nnmail-active-file-coding-system)
+	  (pathname-coding-system 'binary))
+      (nnmail-find-file nnfolder-active-file)
+      (setq nnfolder-group-alist (nnmail-get-active)))
     t))
 
 (deffoo nnfolder-request-newgroups (date &optional server)
@@ -327,80 +329,84 @@ time saver for large mailboxes.")
       (nnmail-save-active nnfolder-group-alist nnfolder-active-file)
       (nconc rest articles))))
 
-(deffoo nnfolder-request-move-article
-  (article group server accept-form &optional last)
-  (let ((buf (get-buffer-create " *nnfolder move*"))
-	result)
-    (and
-     (nnfolder-request-article article group server)
-     (save-excursion
-       (set-buffer buf)
-       (buffer-disable-undo (current-buffer))
-       (erase-buffer)
-       (insert-buffer-substring nntp-server-buffer)
-       (goto-char (point-min))
-       (while (re-search-forward
-	       (concat "^" nnfolder-article-marker)
-	       (save-excursion (search-forward "\n\n" nil t) (point)) t)
-	 (delete-region (progn (beginning-of-line) (point))
-			(progn (forward-line 1) (point))))
-       (setq result (eval accept-form))
-       (kill-buffer buf)
-       result)
-     (save-excursion
-       (nnfolder-possibly-change-group group server)
-       (set-buffer nnfolder-current-buffer)
-       (goto-char (point-min))
-       (when (search-forward (nnfolder-article-string article) nil t)
-	 (nnfolder-delete-mail))
-       (when last
-	 (nnfolder-save-buffer)
-	 (nnfolder-adjust-min-active group)
-	 (nnmail-save-active nnfolder-group-alist nnfolder-active-file))))
-    result))
+(deffoo nnfolder-request-move-article (article group server
+					       accept-form &optional last)
+  (save-excursion
+    (let ((buf (get-buffer-create " *nnfolder move*"))
+	  result)
+      (and
+       (nnfolder-request-article article group server)
+       (save-excursion
+	 (set-buffer buf)
+	 (buffer-disable-undo (current-buffer))
+	 (erase-buffer)
+	 (insert-buffer-substring nntp-server-buffer)
+	 (goto-char (point-min))
+	 (while (re-search-forward
+		 (concat "^" nnfolder-article-marker)
+		 (save-excursion (search-forward "\n\n" nil t) (point)) t)
+	   (delete-region (progn (beginning-of-line) (point))
+			  (progn (forward-line 1) (point))))
+	 (setq result (eval accept-form))
+	 (kill-buffer buf)
+	 result)
+       (save-excursion
+	 (nnfolder-possibly-change-group group server)
+	 (set-buffer nnfolder-current-buffer)
+	 (goto-char (point-min))
+	 (when (search-forward (nnfolder-article-string article) nil t)
+	   (nnfolder-delete-mail))
+	 (when last
+	   (nnfolder-save-buffer)
+	   (nnfolder-adjust-min-active group)
+	   (nnmail-save-active nnfolder-group-alist nnfolder-active-file))))
+      result)))
 
 (deffoo nnfolder-request-accept-article (group &optional server last)
-  (nnfolder-possibly-change-group group server)
-  (nnmail-check-syntax)
-  (let ((buf (current-buffer))
-	result art-group)
-    (goto-char (point-min))
-    (when (looking-at "X-From-Line: ")
-      (replace-match "From "))
-    (and
-     (nnfolder-request-list)
-     (save-excursion
-       (set-buffer buf)
-       (goto-char (point-min))
-       (search-forward "\n\n" nil t)
-       (forward-line -1)
-       (while (re-search-backward (concat "^" nnfolder-article-marker) nil t)
-	 (delete-region (point) (progn (forward-line 1) (point))))
-       (when nnmail-cache-accepted-message-ids
-	 (nnmail-cache-insert (nnmail-fetch-field "message-id")))
-       (setq result (if (stringp group)
-			(list (cons group (nnfolder-active-number group)))
-		      (setq art-group
-			    (nnmail-article-group 'nnfolder-active-number))))
-       (if (and (null result)
-		(yes-or-no-p "Moved to `junk' group; delete article? "))
-	   (setq result 'junk)
-	 (setq result
-	       (car (nnfolder-save-mail result)))))
-     (when last
+  (save-excursion
+    (nnfolder-possibly-change-group group server)
+    (nnmail-check-syntax)
+    (let ((buf (current-buffer))
+	  result art-group)
+      (goto-char (point-min))
+      (when (looking-at "X-From-Line: ")
+	(replace-match "From "))
+      (and
+       (nnfolder-request-list)
        (save-excursion
-	 (nnfolder-possibly-change-folder (or (caar art-group) group))
-	 (nnfolder-save-buffer)
+	 (set-buffer buf)
+	 (goto-char (point-min))
+	 (search-forward "\n\n" nil t)
+	 (forward-line -1)
+	 (while (re-search-backward (concat "^" nnfolder-article-marker) nil t)
+	   (delete-region (point) (progn (forward-line 1) (point))))
 	 (when nnmail-cache-accepted-message-ids
-	   (nnmail-cache-close)))))
-    (nnmail-save-active nnfolder-group-alist nnfolder-active-file)
-    (unless result
-      (nnheader-report 'nnfolder "Couldn't store article"))
-    result))
+	   (nnmail-cache-insert (nnmail-fetch-field "message-id")))
+	 (setq result (if (stringp group)
+			  (list (cons group (nnfolder-active-number group)))
+			(setq art-group
+			      (nnmail-article-group 'nnfolder-active-number))))
+	 (if (and (null result)
+		  (yes-or-no-p "Moved to `junk' group; delete article? "))
+	     (setq result 'junk)
+	   (setq result
+		 (car (nnfolder-save-mail result)))))
+       (when last
+	 (save-excursion
+	   (nnfolder-possibly-change-folder (or (caar art-group) group))
+	   (nnfolder-save-buffer)
+	   (when nnmail-cache-accepted-message-ids
+	     (nnmail-cache-close)))))
+      (nnmail-save-active nnfolder-group-alist nnfolder-active-file)
+      (unless result
+	(nnheader-report 'nnfolder "Couldn't store article"))
+      result)))
 
 (deffoo nnfolder-request-replace-article (article group buffer)
   (nnfolder-possibly-change-group group)
   (save-excursion
+    (set-buffer buffer)
+    (nnfolder-normalize-buffer)
     (set-buffer nnfolder-current-buffer)
     (goto-char (point-min))
     (if (not (search-forward (nnfolder-article-string article) nil t))
@@ -489,9 +495,7 @@ time saver for large mailboxes.")
      (progn
        (forward-line 1)
        (if (nnmail-search-unix-mail-delim)
-	   (if (and (not (bobp)) leave-delim)
-	       (progn (forward-line -2) (point))
-	     (point))
+	   (point)
 	 (point-max))))))
 
 (defun nnfolder-possibly-change-group (group &optional server dont-check)
@@ -505,46 +509,48 @@ time saver for large mailboxes.")
   ;; Change group.
   (when (and group
 	     (not (equal group nnfolder-current-group)))
-    (nnmail-activate 'nnfolder)
-    (when (and (not (assoc group nnfolder-group-alist))
-	       (not (file-exists-p
-		     (nnfolder-group-pathname group))))
-      ;; The group doesn't exist, so we create a new entry for it.
-      (push (list group (cons 1 0)) nnfolder-group-alist)
-      (nnmail-save-active nnfolder-group-alist nnfolder-active-file))
+    (let ((pathname-coding-system 'binary))
+      (nnmail-activate 'nnfolder)
+      (when (and (not (assoc group nnfolder-group-alist))
+		 (not (file-exists-p
+		       (nnfolder-group-pathname group))))
+	;; The group doesn't exist, so we create a new entry for it.
+	(push (list group (cons 1 0)) nnfolder-group-alist)
+	(nnmail-save-active nnfolder-group-alist nnfolder-active-file))
 
-    (if dont-check
-	(setq nnfolder-current-group group
-	      nnfolder-current-buffer nil)
-      (let (inf file)
-	;; If we have to change groups, see if we don't already have the
-	;; folder in memory.  If we do, verify the modtime and destroy
-	;; the folder if needed so we can rescan it.
-	(setq nnfolder-current-buffer
-	      (nth 1 (assoc group nnfolder-buffer-alist)))
+      (if dont-check
+	  (setq nnfolder-current-group group
+		nnfolder-current-buffer nil)
+	(let (inf file)
+	  ;; If we have to change groups, see if we don't already have the
+	  ;; folder in memory.  If we do, verify the modtime and destroy
+	  ;; the folder if needed so we can rescan it.
+	  (setq nnfolder-current-buffer
+		(nth 1 (assoc group nnfolder-buffer-alist)))
 
-	;; If the buffer is not live, make sure it isn't in the alist.  If it
-	;; is live, verify that nobody else has touched the file since last
-	;; time.
-	(when (and nnfolder-current-buffer
-		   (not (gnus-buffer-live-p nnfolder-current-buffer)))
-	  (setq nnfolder-buffer-alist (delq inf nnfolder-buffer-alist)
-		nnfolder-current-buffer nil))
+	  ;; If the buffer is not live, make sure it isn't in the alist.  If it
+	  ;; is live, verify that nobody else has touched the file since last
+	  ;; time.
+	  (when (and nnfolder-current-buffer
+		     (not (gnus-buffer-live-p nnfolder-current-buffer)))
+	    (setq nnfolder-buffer-alist (delq inf nnfolder-buffer-alist)
+		  nnfolder-current-buffer nil))
 
-	(setq nnfolder-current-group group)
+	  (setq nnfolder-current-group group)
 
-	(when (or (not nnfolder-current-buffer)
-		  (not (verify-visited-file-modtime nnfolder-current-buffer)))
-	  (save-excursion
-	    (setq file (nnfolder-group-pathname group))
-	    ;; See whether we need to create the new file.
-	    (unless (file-exists-p file)
-	      (gnus-make-directory (file-name-directory file))
-	      (nnmail-write-region 1 1 file t 'nomesg))
-	    (when (setq nnfolder-current-buffer (nnfolder-read-folder group))
-	      (set-buffer nnfolder-current-buffer)
-	      (push (list group nnfolder-current-buffer)
-		    nnfolder-buffer-alist))))))))
+	  (when (or (not nnfolder-current-buffer)
+		    (not (verify-visited-file-modtime
+			  nnfolder-current-buffer)))
+	    (save-excursion
+	      (setq file (nnfolder-group-pathname group))
+	      ;; See whether we need to create the new file.
+	      (unless (file-exists-p file)
+		(gnus-make-directory (file-name-directory file))
+		(nnmail-write-region 1 1 file t 'nomesg))
+	      (when (setq nnfolder-current-buffer (nnfolder-read-folder group))
+		(set-buffer nnfolder-current-buffer)
+		(push (list group nnfolder-current-buffer)
+		      nnfolder-buffer-alist)))))))))
 
 (defun nnfolder-save-mail (group-art-list)
   "Called narrowed to an article."
@@ -587,16 +593,19 @@ time saver for large mailboxes.")
 	      (obuf (current-buffer)))
 	  (nnfolder-possibly-change-folder (car group-art))
 	  (let ((buffer-read-only nil))
-	    (goto-char (point-max))
-	    (unless (eolp)
-	      (insert "\n"))
-	    (unless (bobp)
-	      (insert "\n"))
+	    (nnfolder-normalize-buffer)
 	    (insert-buffer-substring obuf beg end)))))
 
     ;; Did we save it anywhere?
     save-list))
 
+(defun nnfolder-normalize-buffer ()
+  "Make sure there are two newlines at the end of the buffer."
+  (goto-char (point-max))
+  (skip-chars-backward "\n")
+  (delete-region (point) (point-max))
+  (insert "\n\n"))
+
 (defun nnfolder-insert-newsgroup-line (group-art)
   (save-excursion
     (goto-char (point-min))
@@ -650,7 +659,11 @@ time saver for large mailboxes.")
     (if (equal (cadr (assoc group nnfolder-scantime-alist))
 	       (nth 5 (file-attributes file)))
 	;; This looks up-to-date, so we don't do any scanning.
-	buffer
+	(if (file-exists-p file)
+	    buffer
+	  (push (list group buffer) nnfolder-buffer-alist)
+	  (set-buffer-modified-p t)
+	  (save-buffer))
       ;; Parse the damn thing.
       (save-excursion
 	(nnmail-activate 'nnfolder)
@@ -747,7 +760,9 @@ time saver for large mailboxes.")
           (if oldgroup
               (nnheader-message 5 "Refreshing group %s..." file)
             (nnheader-message 5 "Adding group %s..." file))
-          (setq nnfolder-group-alist (remove oldgroup nnfolder-group-alist))
+	  (if oldgroup
+	      (setq nnfolder-group-alist
+		    (delq oldgroup (copy-sequence nnfolder-group-alist))))
           (push (list file (cons 1 0)) nnfolder-group-alist)
           (nnfolder-possibly-change-folder file)
           (nnfolder-possibly-change-group file)
@@ -756,6 +771,7 @@ time saver for large mailboxes.")
 
 (defun nnfolder-group-pathname (group)
   "Make pathname for GROUP."
+  (setq group (gnus-encode-coding-string group nnmail-pathname-coding-system))
   (let ((dir (file-name-as-directory (expand-file-name nnfolder-directory))))
     ;; If this file exists, we use it directly.
     (if (or nnmail-use-long-file-names
