@@ -1,5 +1,5 @@
 ;;; nntp.el --- nntp access for Gnus
-;;; Copyright (C) 1987,88,89,90,92,93,94,95,96,97 Free Software Foundation, Inc.
+;;; Copyright (C) 1987-90,92-97 Free Software Foundation, Inc.
 
 ;; Author: Lars Magne Ingebrigtsen <larsi@ifi.uio.no>
 ;; Keywords: news
@@ -79,6 +79,10 @@ the NNTP server available there (see nntp-rlogin-parameters) and
 `nntp-open-telnet' which telnets to a remote system, logs in and does
 the same.")
 
+(defvoo nntp-rlogin-program "rsh"
+  "*Program used to log in on remote machines.
+The default is \"rsh\", but \"ssh\" is a popular alternative.")
+
 (defvoo nntp-rlogin-parameters '("telnet" "-8" "${NNTPSERVER:=news}" "nntp")
   "*Parameters to `nntp-open-login'.
 That function may be used as `nntp-open-connection-function'.  In that
@@ -99,6 +103,12 @@ via telnet.")
 (defvoo nntp-telnet-passwd nil
   "Password to use to log in via telnet with.")
 
+(defvoo nntp-open-telnet-envuser nil
+  "*If non-nil, telnet session (client and server both) will support the ENVIRON option and not prompt for login name.")
+
+(defvoo nntp-telnet-shell-prompt "bash\\|\$ *\r?$\\|> *\r?"
+  "*Regular expression to match the shell prompt on the remote machine.")
+
 (defvoo nntp-telnet-command "telnet"
   "Command used to start telnet.")
 
@@ -144,15 +154,42 @@ If can be used to set up a server remotely, for instance.  Say you
 have an account at the machine \"other.machine\".  This machine has
 access to an NNTP server that you can't access locally.  You could
 then use this hook to rsh to the remote machine and start a proxy NNTP
-server there that you can connect to.  See also `nntp-open-connection-function'")
+server there that you can connect to.  See also
+`nntp-open-connection-function'")
 
 (defvoo nntp-warn-about-losing-connection t
   "*If non-nil, beep when a server closes connection.")
 
+(defvoo nntp-coding-system-for-read 'binary
+  "*Coding system to read from NNTP.")
+
+(defvoo nntp-coding-system-for-write 'binary
+  "*Coding system to write to NNTP.")
+
+(defcustom nntp-authinfo-file "~/.authinfo"
+  "Docstring."
+  :type
+  '(choice file
+	   (repeat :tag "Entries"
+		   :menu-tag "Inline"
+		   (list :format "%v"
+			 :value ("" ("login" . "") ("password" . ""))
+			 (string :tag "Host")
+			 (checklist :inline t
+				    (cons :format "%v"
+					  (const :format "" "login")
+					  (string :format "Login: %v"))
+				    (cons :format "%v"
+					  (const :format "" "password")
+					  (string :format "Password: %v")))))))
+
 
 
 ;;; Internal variables.
 
+(defvar nntp-record-commands nil
+  "*If non-nil, nntp will record all commands in the \"*nntp-log*\" buffer.")
+
 (defvar nntp-have-messaged nil)
 
 (defvar nntp-process-wait-for nil)
@@ -161,6 +198,9 @@ server there that you can connect to.  See also `nntp-open-connection-function'"
 (defvar nntp-process-decode nil)
 (defvar nntp-process-start-point nil)
 (defvar nntp-inside-change-function nil)
+(defvoo nntp-last-command-time nil)
+(defvoo nntp-last-command nil)
+(defvoo nntp-authinfo-password nil)
 
 (defvar nntp-connection-list nil)
 
@@ -183,8 +223,23 @@ server there that you can connect to.  See also `nntp-open-connection-function'"
 
 (defsubst nntp-send-string (process string)
   "Send STRING to PROCESS."
+  ;; We need to store the time to provide timeouts, and
+  ;; to store the command so the we can replay the command
+  ;; if the server gives us an AUTHINFO challenge.
+  (setq nntp-last-command-time (current-time)
+	nntp-last-command string)
+  (when nntp-record-commands
+    (nntp-record-command string))
   (process-send-string process (concat string nntp-end-of-line)))
 
+(defun nntp-record-command (string)
+  "Record the command STRING."
+  (save-excursion
+    (set-buffer (get-buffer-create "*nntp-log*"))
+    (goto-char (point-max))
+    (insert (format-time-string "%Y%m%dT%H%M%S" (current-time))
+	    " " nntp-address " " string "\n")))
+
 (defsubst nntp-wait-for (process wait-for buffer &optional decode discard)
   "Wait for WAIT-FOR to arrive from PROCESS."
   (save-excursion
@@ -193,22 +248,25 @@ server there that you can connect to.  See also `nntp-open-connection-function'"
     (while (or (not (memq (char-after (point)) '(?2 ?3 ?4 ?5)))
 	       (looking-at "480"))
       (when (looking-at "480")
-	(erase-buffer)
-	(funcall nntp-authinfo-function))
+	(nntp-handle-authinfo process))
       (nntp-accept-process-output process)
       (goto-char (point-min)))
     (prog1
-	(if (looking-at "[45]")
-	    (progn
-	      (nntp-snarf-error-message)
-	      nil)
+	(cond
+	 ((looking-at "[45]")
+	  (progn
+	    (nntp-snarf-error-message)
+	    nil))
+	 ((not (memq (process-status process) '(open run)))
+	  (nnheader-report 'nntp "Server closed connection"))
+	 (t
 	  (goto-char (point-max))
 	  (let ((limit (point-min)))
 	    (while (not (re-search-backward wait-for limit t))
+	      (nntp-accept-process-output process)
 	      ;; We assume that whatever we wait for is less than 1000
 	      ;; characters long.
 	      (setq limit (max (- (point-max) 1000) (point-min)))
-	      (nntp-accept-process-output process)
 	      (goto-char (point-max))))
 	  (nntp-decode-text (not decode))
 	  (unless discard
@@ -220,7 +278,7 @@ server there that you can connect to.  See also `nntp-open-connection-function'"
 	      (when nntp-have-messaged
 		(setq nntp-have-messaged nil)
 		(message ""))
-	      t)))
+	      t))))
       (unless discard
 	(erase-buffer)))))
 
@@ -252,7 +310,7 @@ server there that you can connect to.  See also `nntp-open-connection-function'"
       (process-buffer process))))
 
 (defsubst nntp-retrieve-data (command address port buffer
-				   &optional wait-for callback decode)
+				      &optional wait-for callback decode)
   "Use COMMAND to retrieve data into BUFFER from PORT on ADDRESS."
   (let ((process (or (nntp-find-connection buffer)
 		     (nntp-open-connection buffer))))
@@ -335,6 +393,20 @@ server there that you can connect to.  See also `nntp-open-connection-function'"
 
 (nnoo-define-basics nntp)
 
+(defsubst nntp-next-result-arrived-p ()
+  (let ((point (point)))
+    (cond
+     ((eq (following-char) ?2)
+      (if (re-search-forward "\n\\.\r?\n" nil t)
+	  t
+	(goto-char point)
+	nil))
+     ((looking-at "[34]")
+      (forward-line 1)
+      t)
+     (t
+      nil))))
+
 (deffoo nntp-retrieve-headers (articles &optional group server fetch-old)
   "Retrieve the headers of ARTICLES."
   (nntp-possibly-change-group group server)
@@ -353,49 +425,39 @@ server there that you can connect to.  See also `nntp-open-connection-function'"
 	    (received 0)
 	    (last-point (point-min))
 	    (buf (nntp-find-connection-buffer nntp-server-buffer))
-	    (nntp-inhibit-erase t))
-	;; Send HEAD command.
-	(while articles
-	  (nntp-send-command
-	   nil
-	   "HEAD" (if (numberp (car articles))
-		      (int-to-string (car articles))
-		    ;; `articles' is either a list of article numbers
-		    ;; or a list of article IDs.
-		    (car articles)))
-	  (setq articles (cdr articles)
-		count (1+ count))
-	  ;; Every 400 header requests we have to read the stream in
-	  ;; order to avoid deadlocks.
-	  (when (or (null articles)	;All requests have been sent.
-		    (zerop (% count nntp-maximum-request)))
-	    (nntp-accept-response)
-	    (while (progn
-		     (progn
-		       (set-buffer buf)
-		       (goto-char last-point))
-		     ;; Count replies.
-		     (while (re-search-forward "^[0-9]" nil t)
-		       (incf received))
-		     (setq last-point (point))
-		     (< received count))
-	      ;; If number of headers is greater than 100, give
-	      ;;  informative messages.
-	      (and (numberp nntp-large-newsgroup)
-		   (> number nntp-large-newsgroup)
-		   (zerop (% received 20))
-		   (nnheader-message 6 "NNTP: Receiving headers... %d%%"
-				     (/ (* received 100) number)))
-	      (nntp-accept-response))))
-	;; Wait for text of last command.
-	(goto-char (point-max))
-	(re-search-backward "^[0-9]" nil t)
-	(when (looking-at "^[23]")
+	    (nntp-inhibit-erase t)
+	    article)
+	;; Send HEAD commands.
+      (while (setq article (pop articles))
+	(nntp-send-command
+	 nil
+	 "HEAD" (if (numberp article)
+		    (int-to-string article)
+		  ;; `articles' is either a list of article numbers
+		  ;; or a list of article IDs.
+		  article))
+	(incf count)
+	;; Every 400 requests we have to read the stream in
+	;; order to avoid deadlocks.
+	(when (or (null articles)	;All requests have been sent.
+		  (zerop (% count nntp-maximum-request)))
+	  (nntp-accept-response)
 	  (while (progn
-		   (goto-char (point-max))
-		   (forward-line -1)
-		   (not (looking-at "^\\.\r?\n")))
-	    (nntp-accept-response)))
+		   (set-buffer buf)
+		   (goto-char last-point)
+		   ;; Count replies.
+		   (while (nntp-next-result-arrived-p)
+		     (setq last-point (point))
+		     (incf received))
+		   (< received count))
+	    ;; If number of headers is greater than 100, give
+	    ;;  informative messages.
+	    (and (numberp nntp-large-newsgroup)
+		 (> number nntp-large-newsgroup)
+		 (zerop (% received 20))
+		 (nnheader-message 6 "NNTP: Receiving headers... %d%%"
+				   (/ (* received 100) number)))
+	    (nntp-accept-response))))
 	(and (numberp nntp-large-newsgroup)
 	     (> number nntp-large-newsgroup)
 	     (nnheader-message 6 "NNTP: Receiving headers...done"))
@@ -483,7 +545,7 @@ server there that you can connect to.  See also `nntp-open-connection-function'"
 	  article alist)
       (set-buffer buf)
       (erase-buffer)
-      ;; Send HEAD command.
+      ;; Send ARTICLE command.
       (while (setq article (pop articles))
 	(nntp-send-command
 	 nil
@@ -499,14 +561,13 @@ server there that you can connect to.  See also `nntp-open-connection-function'"
 		  (zerop (% count nntp-maximum-request)))
 	  (nntp-accept-response)
 	  (while (progn
-		   (progn
-		     (set-buffer buf)
-		     (goto-char last-point))
+		   (set-buffer buf)
+		   (goto-char last-point)
 		   ;; Count replies.
 		   (while (nntp-next-result-arrived-p)
 		     (aset map received (cons (aref map received) (point)))
+		     (setq last-point (point))
 		     (incf received))
-		   (setq last-point (point))
 		   (< received count))
 	    ;; If number of headers is greater than 100, give
 	    ;;  informative messages.
@@ -518,12 +579,13 @@ server there that you can connect to.  See also `nntp-open-connection-function'"
 	    (nntp-accept-response))))
       (and (numberp nntp-large-newsgroup)
 	   (> number nntp-large-newsgroup)
-	   (nnheader-message 6 "NNTP: Receiving headers...done"))
-
+	   (nnheader-message 6 "NNTP: Receiving articles...done"))
+      
       ;; Now we have all the responses.  We go through the results,
       ;; washes it and copies it over to the server buffer.
       (set-buffer nntp-server-buffer)
       (erase-buffer)
+      (setq last-point (point-min))
       (mapcar
        (lambda (entry)
 	 (narrow-to-region
@@ -531,25 +593,12 @@ server there that you can connect to.  See also `nntp-open-connection-function'"
 	  (progn
 	    (insert-buffer-substring buf last-point (cdr entry))
 	    (point-max)))
+	 (setq last-point (cdr entry))
 	 (nntp-decode-text)
 	 (widen)
 	 (cons (car entry) point))
        map))))
 
-(defun nntp-next-result-arrived-p ()
-  (let ((point (point)))
-    (cond
-     ((looking-at "2")
-      (if (re-search-forward "\n.\r?\n" nil t)
-	  t
-	(goto-char point)
-	nil))
-     ((looking-at "[34]")
-      (forward-line 1)
-      t)
-     (t
-      nil))))
-
 (defun nntp-try-list-active (group)
   (nntp-list-active-group group)
   (save-excursion
@@ -596,7 +645,7 @@ server there that you can connect to.  See also `nntp-open-connection-function'"
 
 (deffoo nntp-request-group (group &optional server dont-check)
   (nntp-possibly-change-group nil server)
-  (when (nntp-send-command "^2.*\n" "GROUP" group)
+  (when (nntp-send-command "^21.*\n" "GROUP" group)
     (let ((entry (nntp-find-connection-entry nntp-server-buffer)))
       (setcar (cddr entry) group))))
 
@@ -629,8 +678,10 @@ server there that you can connect to.  See also `nntp-open-connection-function'"
   (let (process)
     (while (setq process (car (pop nntp-connection-alist)))
       (when (memq (process-status process) '(open run))
-	(set-process-sentinel process nil)
-	(nntp-send-string process "QUIT"))
+	(ignore-errors
+	  (nntp-send-string process "QUIT")
+	  (unless (eq nntp-open-connection-function 'nntp-open-network-stream)
+	    (sleep-for 1))))
       (when (buffer-name (process-buffer process))
 	(kill-buffer (process-buffer process))))
     (nnoo-close-server 'nntp)))
@@ -639,9 +690,14 @@ server there that you can connect to.  See also `nntp-open-connection-function'"
   (let (process)
     (while (setq process (pop nntp-connection-list))
       (when (memq (process-status process) '(open run))
-	(set-process-sentinel process nil)
 	(ignore-errors
-	  (nntp-send-string process "QUIT")))
+	  (nntp-send-string process "QUIT")
+	  (unless (eq nntp-open-connection-function 'nntp-open-network-stream)
+	    ;; Ok, this is evil, but when using telnet and stuff
+	    ;; as the connection method, it's important that the
+	    ;; QUIT command actually is sent out before we kill
+	    ;; the process.  
+	    (sleep-for 1))))
       (when (buffer-name (process-buffer process))
 	(kill-buffer (process-buffer process))))))
 
@@ -688,40 +744,66 @@ It will make innd servers spawn an nnrpd process to allow actual article
 reading."
   (nntp-send-command "^.*\r?\n" "MODE READER"))
 
+(defun nntp-send-authinfo ()
+  "Send the AUTHINFO to the nntp server.
+This function is supposed to be called from `nntp-server-opened-hook'.
+It will look in the \"~/.authinfo\" file for matching entries.  If
+nothing suitable is found there, it will prompt for a user name
+and a password."
+  (let* ((list (gnus-parse-netrc nntp-authinfo-file))
+	 (alist (gnus-netrc-machine list nntp-address))
+	 (user (gnus-netrc-get alist "login"))
+	 (passwd (gnus-netrc-get alist "password")))
+    (nntp-send-command
+     "^3.*\r?\n" "AUTHINFO USER"
+     (or user (read-string (format "NNTP (%s) user name: " nntp-address))))
+    (nntp-send-command
+     "^2.*\r?\n" "AUTHINFO PASS"
+     (or passwd
+	 nntp-authinfo-password
+	 (setq nntp-authinfo-password
+	       (nnmail-read-passwd (format "NNTP (%s) password: "
+					   nntp-address)))))))
+
 (defun nntp-send-nosy-authinfo ()
   "Send the AUTHINFO to the nntp server.
 This function is supposed to be called from `nntp-server-opened-hook'.
 It will prompt for a password."
   (nntp-send-command
-   "^.*\r?\n" "AUTHINFO USER"
+   "^3.*\r?\n" "AUTHINFO USER"
    (read-string (format "NNTP (%s) user name: " nntp-address)))
   (nntp-send-command
-   "^.*\r?\n" "AUTHINFO PASS"
+   "^2.*\r?\n" "AUTHINFO PASS"
    (nnmail-read-passwd "NNTP (%s) password: " nntp-address)))
 
-(defun nntp-send-authinfo ()
-  "Send the AUTHINFO to the nntp server.
-This function is supposed to be called from `nntp-server-opened-hook'.
-It will prompt for a password."
-  (nntp-send-command "^.*\r?\n" "AUTHINFO USER" (user-login-name))
-  (nntp-send-command
-   "^.*\r?\n" "AUTHINFO PASS"
-   (nnmail-read-passwd (format "NNTP (%s) password: " nntp-address))))
-
 (defun nntp-send-authinfo-from-file ()
   "Send the AUTHINFO to the nntp server.
-This function is supposed to be called from `nntp-server-opened-hook'."
+This function is supposed to be called from `nntp-server-opened-hook'.
+
+The authinfo login name is taken from the user's login name and the
+password contained in '~/.nntp-authinfo'."
   (when (file-exists-p "~/.nntp-authinfo")
     (nnheader-temp-write nil
       (insert-file-contents "~/.nntp-authinfo")
       (goto-char (point-min))
-      (nntp-send-command "^.*\r?\n" "AUTHINFO USER" (user-login-name))
+      (nntp-send-command "^3.*\r?\n" "AUTHINFO USER" (user-login-name))
       (nntp-send-command
-       "^.*\r?\n" "AUTHINFO PASS"
+       "^2.*\r?\n" "AUTHINFO PASS"
        (buffer-substring (point) (progn (end-of-line) (point)))))))
 
 ;;; Internal functions.
 
+(defun nntp-handle-authinfo (process)
+  "Take care of an authinfo response from the server."
+  (let ((last nntp-last-command))
+    (funcall nntp-authinfo-function)
+    ;; We have to re-send the function that was interrupted by
+    ;; the authinfo request.
+    (save-excursion
+      (set-buffer nntp-server-buffer)
+      (erase-buffer))
+    (nntp-send-string process last)))
+
 (defun nntp-make-process-buffer (buffer)
   "Create a new, fresh buffer usable for nntp process connections."
   (save-excursion
@@ -745,7 +827,8 @@ This function is supposed to be called from `nntp-server-opened-hook'."
   (let* ((pbuffer (nntp-make-process-buffer buffer))
 	 (process
 	  (condition-case ()
-	      (funcall nntp-open-connection-function pbuffer)
+	      (let ((coding-system-for-read nntp-coding-system-for-read))
+		(funcall nntp-open-connection-function pbuffer))
 	    (error nil)
 	    (quit nil))))
     (when process
@@ -794,7 +877,7 @@ This function is supposed to be called from `nntp-server-opened-hook'."
 	  (save-excursion
 	    (goto-char beg)
 	    (if (looking-at "480")
-		(funcall nntp-authinfo-function)
+		(nntp-handle-authinfo nntp-process-to-buffer)
 	      (nntp-snarf-error-message)
 	      (funcall nntp-process-callback nil)))
 	(goto-char end)
@@ -896,7 +979,12 @@ This function is supposed to be called from `nntp-server-opened-hook'."
       (insert "\n"))
     ;; Insert `.' at end of buffer (end of text mark).
     (goto-char (point-max))
-    (insert "." nntp-end-of-line)))
+    (insert ".\n")
+    (goto-char (point-min))
+    (while (not (eobp))
+      (end-of-line)
+      (insert "\r")
+      (forward-line 1))))
 
 (defun nntp-retrieve-headers-with-xover (articles &optional fetch-old)
   (set-buffer nntp-server-buffer)
@@ -1055,13 +1143,20 @@ This function is supposed to be called from `nntp-server-opened-hook'."
 	  (case-fold-search t))
       (when (memq (process-status proc) '(open run))
 	(process-send-string proc "set escape \^X\n")
-	(process-send-string proc (concat "open " nntp-address "\n"))
-	(nntp-wait-for-string "^\r*.?login:")
-	(process-send-string
-	 proc (concat
-	       (or nntp-telnet-user-name
-		   (setq nntp-telnet-user-name (read-string "login: ")))
-	       "\n"))
+	(cond
+	 ((and nntp-open-telnet-envuser nntp-telnet-user-name)
+	  (process-send-string proc (concat "open " "-l" nntp-telnet-user-name
+					    nntp-address "\n")))
+	 (t
+	  (process-send-string proc (concat "open " nntp-address "\n"))))
+	(cond
+	 ((not nntp-open-telnet-envuser)
+	  (nntp-wait-for-string "^\r*.?login:")
+	  (process-send-string
+	   proc (concat
+		 (or nntp-telnet-user-name
+		     (setq nntp-telnet-user-name (read-string "login: ")))
+		 "\n"))))
 	(nntp-wait-for-string "^\r*.?password:")
 	(process-send-string
 	 proc (concat
@@ -1070,10 +1165,10 @@ This function is supposed to be called from `nntp-server-opened-hook'."
 			 (nnmail-read-passwd "Password: ")))
 	       "\n"))
 	(erase-buffer)
-	(nntp-wait-for-string "bash\\|\$ *\r?$\\|> *\r?")
+	(nntp-wait-for-string nntp-telnet-shell-prompt)
 	(process-send-string
 	 proc (concat (mapconcat 'identity nntp-telnet-parameters " ") "\n"))
-	(nntp-wait-for-string "^\r*200")
+	(nntp-wait-for-string "^\r*20[01]")
 	(beginning-of-line)
 	(delete-region (point-min) (point))
 	(process-send-string proc "\^]")
@@ -1089,17 +1184,15 @@ This function is supposed to be called from `nntp-server-opened-hook'."
 (defun nntp-open-rlogin (buffer)
   "Open a connection to SERVER using rsh."
   (let ((proc (if nntp-rlogin-user-name
-		  (start-process
-		   "nntpd" buffer "rsh"
-		   nntp-address "-l" nntp-rlogin-user-name
-		   (mapconcat 'identity
-			      nntp-rlogin-parameters " "))
-		(start-process
-		 "nntpd" buffer "rsh" nntp-address
-		 (mapconcat 'identity
-			    nntp-rlogin-parameters " ")))))
+		  (apply 'start-process
+			 "nntpd" buffer nntp-rlogin-program
+			 nntp-address "-l" nntp-rlogin-user-name
+			 nntp-rlogin-parameters)
+		(apply 'start-process
+		       "nntpd" buffer nntp-rlogin-program nntp-address
+		       nntp-rlogin-parameters))))
     (set-buffer buffer)
-    (nntp-wait-for-string "^\r*200")
+    (nntp-wait-for-string "^\r*20[01]")
     (beginning-of-line)
     (delete-region (point-min) (point))
     proc))
