@@ -1,5 +1,5 @@
 ;;; nnvirtual.el --- virtual newsgroups access for Gnus
-;; Copyright (C) 1994,95,96,97 Free Software Foundation, Inc.
+;; Copyright (C) 1994,95,96,97,98 Free Software Foundation, Inc.
 
 ;; Author: David Moore <dmoore@ucsd.edu>
 ;;	Lars Magne Ingebrigtsen <larsi@ifi.uio.no>
@@ -38,7 +38,8 @@
 (require 'gnus-util)
 (require 'gnus-start)
 (require 'gnus-sum)
-(eval-when-compile (require 'cl))
+(require 'gnus-msg)
+(require 'cl)
 
 (nnoo-declare nnvirtual)
 
@@ -342,6 +343,15 @@ to virtual article number.")
   "Return the real group and article for virtual GROUP and ARTICLE."
   (nnvirtual-map-article article))
 
+
+(deffoo nnvirtual-request-post (&optional server)
+  (if (not gnus-message-group-art)
+      (nnheader-report 'nnvirtual "Can't post to an nnvirtual group")
+    (let ((group (car (nnvirtual-find-group-art
+		       (car gnus-message-group-art)
+		       (cdr gnus-message-group-art)))))
+      (gnus-request-post (gnus-find-method-for-group group)))))
+
 
 ;;; Internal functions.
 
@@ -472,10 +482,7 @@ If UPDATE-P is not nil, call gnus-group-update-group on the components."
   "Merge many sorted lists of numbers."
   (if (null (cdr lists))
       (car lists)
-    (apply 'nnvirtual-merge-sorted-lists
-	   (merge 'list (car lists) (cadr lists) '<)
-	   (cddr lists))))
-
+    (sort (apply 'nconc lists) '<)))
 
 
 ;;; We map between virtual articles and real articles in a manner
@@ -563,27 +570,28 @@ If UPDATE-P is not nil, call gnus-group-update-group on the components."
 
 (defun nnvirtual-reverse-map-article (group article)
   "Return the virtual article number corresponding to the given component GROUP and ARTICLE."
-  (let ((table nnvirtual-mapping-table)
-	(group-pos 0)
-	entry)
-    (while (not (string= group (car (aref nnvirtual-mapping-offsets
+  (when (numberp article)
+    (let ((table nnvirtual-mapping-table)
+	  (group-pos 0)
+	  entry)
+      (while (not (string= group (car (aref nnvirtual-mapping-offsets
+					    group-pos))))
+	(setq group-pos (1+ group-pos)))
+      (setq article (- article (cdr (aref nnvirtual-mapping-offsets
 					  group-pos))))
-      (setq group-pos (1+ group-pos)))
-    (setq article (- article (cdr (aref nnvirtual-mapping-offsets
-					group-pos))))
-    (while (and table
-		(> article (aref (car table) 0)))
-      (setq table (cdr table)))
-    (setq entry (car table))
-    (when (and entry
-	       (> article 0)
-	       (< group-pos (aref entry 2))) ; article not out of range below
-      (+ (aref entry 4)
-	 group-pos
-	 (* (- article (aref entry 1))
-	    (aref entry 2))
-	 1))
-    ))
+      (while (and table
+		  (> article (aref (car table) 0)))
+	(setq table (cdr table)))
+      (setq entry (car table))
+      (when (and entry
+		 (> article 0)
+		 (< group-pos (aref entry 2))) ; article not out of range below
+	(+ (aref entry 4)
+	   group-pos
+	   (* (- article (aref entry 1))
+	      (aref entry 2))
+	   1))
+      )))
 
 
 (defsubst nnvirtual-reverse-map-sequence (group articles)
@@ -633,8 +641,8 @@ the result."
 	  (setq entry (assoc (car article) carticles))
 	  (setcdr entry (cons (cdr article) (cdr entry))))
 	(setq i (1+ i))))
-    (mapc (lambda (x) (setcdr x (nreverse (cdr x))))
-	  carticles)
+    (mapcar (lambda (x) (setcdr x (nreverse (cdr x))))
+	    carticles)
     carticles))
 
 
@@ -739,7 +747,11 @@ based on the marks on the component groups."
 		 gnus-article-mark-lists))
 
     ;; Remove any empty marks lists, and store.
-    (setq nnvirtual-mapping-marks (delete-if-not 'cdr marks))
+    (setq nnvirtual-mapping-marks nil)
+    (while marks
+      (if (cdr (car marks))
+	  (push (car marks) nnvirtual-mapping-marks))
+      (setq marks (cdr marks)))
 
     ;; We need to convert the unreads to reads.  We compress the
     ;; sequence as we go, otherwise it could be huge.
